/*
 * Copyright 2016 Boshra Rajaei <b.rajaei@sadjad.ac.ir>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "prSAMP_Calibration.h"


int main(int argc, char *argv[])
{
  int i, j;
  double *p_prms, eps;
  double damp, vnf;
  double delta, *y;
  int *ir, *jc;
  int m, n, p, nnz; //nnz: number of non zero cells in X
  int disp, t_max;
  time_t t1, t2;
  char part[5] = "0";

  if(argc>1)
    strcpy(part, argv[1]);


  FILE *fp,*fp_w;
  char name[30] = "input";
  strcat(name, part);
  strcat(name, ".txt");
  fp = fopen(name, "r");


  readDimensions(fp, &m, &n, &p, &nnz);

  p_prms = malloc(3 * sizeof *p_prms);
  ir = malloc(nnz*sizeof(*ir));
  jc = malloc((n+1)*sizeof(*jc));

  readFromText(fp, n, nnz, ir, jc, p_prms, &t_max, &eps, &damp, &vnf, &delta, &disp);

  complex double *a;

  int  my_rank, num_procs, sender, max_rows_per_process, an_id;
  double avg_rows_per_process;
  int end_row, num_rows_rec;
  MPI_Status status;
  int root_process = 0;

  //time_t t1, t2;
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);
  MPI_Comm_size(MPI_COMM_WORLD, &num_procs);
  int start_row[num_procs], num_rows[num_procs];
  if(num_procs>m) num_procs = m;

  if(my_rank==0)//master
  {
      t1 = time(NULL);
      y = malloc(p*m* sizeof(*y));
      readFromTextY(fp, m, p, y);
      fclose(fp);

    srand(time(NULL));
    gaussrand();
    complex double *h = malloc(m*n*sizeof(*h));
    for(j=0;j<m*n;j++)
         h[j] = gaussrand()*sqrt(1.0/n)+gaussrand()*sqrt(1.0/n)*I;



      avg_rows_per_process = (double) m / num_procs;
      max_rows_per_process = round(avg_rows_per_process)+1;

      printf("I am master! :-) we are %d processors. We will solve prblmid: %s.\n", num_procs, part);
      printf("Average rows per processor is: %lf.\n", avg_rows_per_process);

      start_row[0] = 0;
      end_row = round(avg_rows_per_process) - 1;


      for(an_id = 1; an_id < num_procs; an_id++) {
            start_row[an_id] = end_row + 1;
            end_row   = round((an_id + 1)*avg_rows_per_process)-1;

            if(an_id == num_procs-1){
               end_row = m - 1;
               max_rows_per_process = max(max_rows_per_process, end_row - start_row[an_id] + 1);
               }

            num_rows[an_id] = end_row - start_row[an_id] + 1;

            MPI_Send( &num_rows[an_id], 1 , MPI_INT,
                  an_id, send_data_tag, MPI_COMM_WORLD);

            MPI_Send( &h[n*start_row[an_id]], n*num_rows[an_id], MPI_C_DOUBLE_COMPLEX,
                  an_id, send_data_tag, MPI_COMM_WORLD);

            MPI_Send( &y[p*start_row[an_id]], p*num_rows[an_id], MPI_DOUBLE,
                  an_id, send_data_tag, MPI_COMM_WORLD);

         }


      for(i = 0; i < avg_rows_per_process + 1; i++) {
            cgamp(n, p, i*p, i*n, y, ir, jc, delta, p_prms,
                t_max, eps, damp, vnf, disp, h);
            //printf("%d is done! \n", i);
         }

      a = malloc(max_rows_per_process*n*sizeof(*a));
      for(an_id = 1; an_id < num_procs; an_id++) {

          MPI_Recv( &a[0], max_rows_per_process*n, MPI_C_DOUBLE_COMPLEX, MPI_ANY_SOURCE,
                  return_data_tag, MPI_COMM_WORLD, &status);

            sender = status.MPI_SOURCE;

            for(i=0;i<n*num_rows[sender];i++)
                h[n*start_row[sender]+i] = a[i];

            //printf("process %i done! \n", sender);

         }

    char nameo[30] = "output";
    strcat(nameo, part);
    strcat(nameo, ".txt");
    fp_w = fopen(nameo, "w");
    writeRow(fp_w, m*n, h);
    fclose(fp_w);
    t2 = time(NULL);
    //printf("execution time: %d\n", t2-t1);

  }
  else
  {
      fclose(fp);
      MPI_Recv( &num_rows_rec, 1, MPI_INT,
               root_process, send_data_tag, MPI_COMM_WORLD, &status);

      a = malloc(num_rows_rec*n*sizeof(*a));
      y = malloc(num_rows_rec*p*sizeof(*y));

      MPI_Recv( &a[0], n*num_rows_rec, MPI_C_DOUBLE_COMPLEX,
               root_process, send_data_tag, MPI_COMM_WORLD, &status);
    MPI_Recv( &y[0], p*num_rows_rec, MPI_DOUBLE,
               root_process, send_data_tag, MPI_COMM_WORLD, &status);

      //printf("I am slave %d! :-)\n", my_rank);

      for (i=0; i<num_rows_rec; ++i)
      {
          cgamp(n, p, i*p, i*n, y, ir, jc, delta, p_prms,
            t_max, eps, damp, vnf, disp, a);
            //printf("%d is done! \n", i);
      }
      MPI_Send( &a[0], n*num_rows_rec, MPI_C_DOUBLE_COMPLEX, root_process,
               return_data_tag, MPI_COMM_WORLD);
  }


  MPI_Finalize();

  return 0;
}
